-- Makes an MO exhibit more bouncy bounce mechanics, combined with the fermat's spiral mine
--
-- Fermat's Spiral notes:
-- radius = spiralScale * squareroot( pountCount )
-- radius / squareroot( pountCount ) = spiralScale

function Create(self)

   -- bounce variables
   self.bounceVelThreshold = 1; -- minimum velocity the MO needs to be going to trigger the Lua'd bouncing
   self.rayDistance = 4; -- how far to the "side" of the MO the angle detector rays are
   self.bounce = 0.95; -- how much energy to maintain after bouncing. 1 is 100% energy, 0.9 is 90%, etc.

   -- Fermat's Spiral variables
   self.pointCount = 100; -- number of points in the spiral
   self.spiralScale = 5; -- spiral scale
   self.skipPoints = 10; -- skips the first X points

   -- bomb variables
   self.detTime = 5000; -- time it takes for the grenade to detonate on its own in MS
   self.detTimer = Timer();

end

function Update(self)

   self.ToDelete = false;

   -- bouncy ray stuff
   local rayHitA = Vector();
   local rayHitB = Vector();
   local raylength = Vector(self.Vel.X,self.Vel.Y):SetMagnitude(Vector(self.Vel.X,self.Vel.Y).Magnitude+self.Radius*self.rayDistance+50)*TimerMan.DeltaTimeSecs*20;
   local startA = self.Pos+Vector(0,self.rayDistance):RadRotate(self.Vel.AbsRadAngle);
   local startB = self.Pos+Vector(0,-self.rayDistance):RadRotate(self.Vel.AbsRadAngle);
   local rayA = SceneMan:CastStrengthRay(startA,raylength,0,rayHitA,1,0,SceneMan.SceneWrapsX);
   local rayB = SceneMan:CastStrengthRay(startB,raylength,0,rayHitB,1,0,SceneMan.SceneWrapsX);
   if rayA and rayB then
      local distA = SceneMan:ShortestDistance(startA,rayHitA,SceneMan.SceneWrapsX).Magnitude;
      local distB = SceneMan:ShortestDistance(startB,rayHitB,SceneMan.SceneWrapsX).Magnitude;
      if distA < self.Radius*2 or distB < self.Radius*2 then
         if distA < distB then
            local newVel = SceneMan:ShortestDistance(rayHitA,rayHitB,SceneMan.SceneWrapsX);
            newVel:RadRotate(newVel.AbsRadAngle-self.Vel.AbsRadAngle);
            self.Vel = newVel:SetMagnitude(self.Vel.Magnitude*self.bounce);
         else
            local newVel = SceneMan:ShortestDistance(rayHitB,rayHitA,SceneMan.SceneWrapsX);
            newVel:RadRotate(newVel.AbsRadAngle-self.Vel.AbsRadAngle);
            self.Vel = newVel:SetMagnitude(self.Vel.Magnitude*self.bounce);
         end
      end
   end

   --Fermat's Spiral
   if SceneMan.SceneWrapsX then
      local angle = 0;
      for i = self.skipPoints, self.pointCount - 1 do
         local radius = self.spiralScale*math.sqrt(i);
         angle = angle + 137.508;
         local checkPos = self.Pos + Vector(radius,0):DegRotate(angle);
         if checkPos.X > SceneMan.SceneWidth then
            checkPos = Vector(checkPos.X - SceneMan.SceneWidth,checkPos.Y);
         elseif checkPos.X < 0 then
            checkPos = Vector(SceneMan.SceneWidth + checkPos.X,checkPos.Y);
         end
         local moCheck = SceneMan:GetMOIDPixel(checkPos.X,checkPos.Y);
         if moCheck ~= 255 then
            local actor = MovableMan:GetMOFromID( MovableMan:GetMOFromID(moCheck).RootID );
            if MovableMan:IsActor(actor) and actor.Team ~= self.Team then
               self:GibThis();
               break;
            end
         end
      end
   else
      local angle = 0;
      for i = self.skipPoints, self.pointCount - 1 do
         local radius = self.spiralScale*math.sqrt(i);
         angle = angle + 137.508;
         local checkPos = self.Pos + Vector(radius,0):DegRotate(angle);
         local moCheck = SceneMan:GetMOIDPixel(checkPos.X,checkPos.Y);
         if moCheck ~= 255 then
            local actor = MovableMan:GetMOFromID( MovableMan:GetMOFromID(moCheck).RootID );
            if MovableMan:IsActor(actor) and actor.Team ~= self.Team then
               self:GibThis();
               break;
            end
         end
      end
   end

   -- bomb
   if self.detTimer:IsPastSimMS(self.detTime) then
      self:GibThis();
   end

end