function ArcaMissileLauncherDrawRotating(array,offset,rotation,pos,color)
	for i = 1, #array-1 do
		local v1 = (array[i]+offset):RadRotate(rotation);
		local v2 = (array[i+1]+offset):RadRotate(rotation);
		FrameMan:DrawLinePrimitive(v1+pos,v2+pos,color);
	end
end

function Create(self)

	self.scanTimer = Timer();
	self.scanTime = 500; -- time it takes to scan

	self.startScanTimer = Timer();
	self.startScanTime = 500; -- time it takes to begin scanning

	self.scanRange = 1750;
	self.scanSpacing = 5; -- pixel spacing for the MORay

	self.fireVel = 15;

	self.startFire = false;
	self.startScan = false;

	self.aimEffect = {Vector(0,5), Vector(5,0), Vector(10,5), Vector(10,6), Vector(5,1), Vector(0,6), Vector(1,6), Vector(5,2), Vector(9,6)};
	self.effectTimer = Timer();
	self.effectRotPerSec = 1;
	self.effectOffset = Vector(-6,-14);

end

function Update(self)

	local turnthing = self.effectTimer.ElapsedSimTimeMS/1000;
	if self.effectTimer:IsPastSimMS(1000) then
		self.effectTimer:Reset();
		turnthing = turnthing - math.floor(turnthing);
	end
	self.effectAngle = math.pi*2*turnthing;

	if self.Magazine ~= null and self.Magazine.RoundCount > 0 then
		if self:IsActivated() then
			if self.startFire then

				if self.startScanTimer:IsPastSimMS(self.startScanTime) then

					if not self.startScan then
						self.startScan = true;
						self.scanTimer:Reset();

						local sfx = CreateAEmitter("GDLAW Aim SFX Emitter");
						sfx.Pos = self.Pos;
						MovableMan:AddParticle(sfx);
					end

					local negativenum = 1;
					if self.HFlipped then
						negativenum = -1;
					end
					local mo = SceneMan:CastMORay(self.MuzzlePos, Vector(self.scanRange*negativenum,0):RadRotate(self.RotAngle), self.RootID, self.Team, 0, false, self.scanSpacing);
					if mo ~= 255 then
						local actor = MovableMan:GetMOFromID(MovableMan:GetMOFromID(mo).RootID);
						if MovableMan:IsActor(actor) then
							if self.maybeTarget == null then
								self.maybeTarget = actor.UniqueID;
								self.target = null;
								self.scanTimer:Reset();
							else
								if self.maybeTarget == actor.UniqueID then
									if self.scanTimer:IsPastSimMS(self.scanTime) then
										if self.target == null then
											self.target = self.maybeTarget;
											self.targetActor = actor;
	
											local sfx = CreateAEmitter("GDLAW Lock SFX Emitter");
											sfx.Pos = self.Pos;
											MovableMan:AddParticle(sfx);
										end
									else
										for i = 1, 4 do
											local angle = self.effectAngle+(math.pi*0.5*i);
											local soffset = Vector(0,(100 - 100 * math.min(1,(self.scanTimer.ElapsedSimTimeMS/self.scanTime)))):RadRotate(angle);
											ArcaMissileLauncherDrawRotating(self.aimEffect, self.effectOffset, angle+math.pi, soffset+actor.Pos, 120);
										end
									end

								else
									self.scanTimer:Reset();
									self.maybeTarget = actor.UniqueID;
									self.target = null;
									self.targetActor = null
								end
							end
						else
							if self.target == null then
								self.scanTimer:Reset();
								self.maybeTarget = null;
								self.targetActor = null;
							end
						end
					else
						if self.target == null then
							self.scanTimer:Reset();
							self.maybeTarget = null;
							self.targetActor = null;
						end
					end
				end

				if self.targetActor ~= null then
					if MovableMan:IsActor(self.targetActor) then
						for i = 1, 4 do
							local angle = self.effectAngle+(math.pi*0.5*i);
							local soffset = Vector(0,(100 - 100 * math.min(1,(self.scanTimer.ElapsedSimTimeMS/self.scanTime)))):RadRotate(angle);
							ArcaMissileLauncherDrawRotating(self.aimEffect, self.effectOffset, angle+math.pi, soffset+self.targetActor.Pos, 120);
						end
					else
						self.scanTimer:Reset();
						self.maybeTarget = null;
						self.target = null;
						self.targetActor = null
					end
				end

			else
				self.startScanTimer:Reset();
				self.startScan = false;
				self.maybeTarget = null;
				self.target = null;
				self.targetActor = null;
			end
			self.startFire = true;
		else
			if self.startFire then
				local negativenum = 1;
				if self.HFlipped then
					negativenum = -1;
				end

				local missile = "";

				if self.Sharpness == 0 then
					missile = CreateAEmitter("GDLAW Missile Active");
				end

				missile.Pos = self.MuzzlePos;
				missile.Vel = self.Vel + Vector(self.fireVel*negativenum,0):RadRotate(self.RotAngle);
				missile.RotAngle = missile.Vel.AbsRadAngle;
				missile.Team = self.Team;
				missile.IgnoresTeamHits = true;
				if self.target ~= null then
					missile.Sharpness = self.target;
				end
				MovableMan:AddParticle(missile);

				local sfx = CreateAEmitter("GDLAW Fire SFX Emitter");
				sfx.Pos = self.Pos;
				MovableMan:AddParticle(sfx);
				
				local backblast = CreateAEmitter("GDLAW Missile Backblast");
				backblast.Pos = self.Pos + Vector(-16*negativenum,0):RadRotate(self.RotAngle); -- backblast offset
				backblast.Vel = self.Vel + Vector(-5*negativenum,0):RadRotate(self.RotAngle); -- backblast velocity
				MovableMan:AddParticle(backblast);
				
				self.Magazine.RoundCount = self.Magazine.RoundCount - 1;

				if self.Magazine.RoundCount <= 0 then
					self:Reload();
				end
			end
			self.startFire = false;
		end
	end
end